#!/bin/bash

# File:         redirection.sh
# Name:         Kevin Crotty
# Date:         21 Mar 2019
# Desc:         Redirction in Bash Demo's
# Usage:        ./redirection.sh

# Set Colors for use in the script
export NC='\e[0m'               # Text Reset
export BBlack='\e[1;30m'        # Black
export BRed='\e[1;31m'          # Red
export BGreen='\e[1;32m'        # Green
export BYellow='\e[1;33m'       # Yellow
export BBlue='\e[1;34m'         # Blue
export BPurple='\e[1;35m'       # Purple
export BCyan='\e[1;36m'         # Cyan
export BWhite='\e[1;37m'        # White

function colorComment { echo -e "\n${BGreen}${1}${NC}"
}
function colorCommand { echo -e "${BRed}${1}${NC}"
}

out_demo ()
{
    reset
    colorComment "# Redirection - Overwrite Destination File"
    echo -e "${BRed}echo \"This is some standard output\" > out_1.txt${NC}"
    echo -e "${BRed}cat out_1.txt${NC}"
    echo -e "This is some standard output"

    colorComment "# Redirection - Only writes to last file redirected to (still overwrites out_1.txt) "
    echo -e "${BRed}echo \"This is some more standard output\" > out_1.txt > out_2.Text${NC}"
    echo -e "${BRed}cat out_1.txt${NC}"
    echo -e "${BRed}cat out_2.txt${NC}"
    echo -e "This is some more standard output"

}

in_demo ()
{
    reset
    colorComment "# Make file"
    echo -e "${BRed}echo \"The first line of in_1.txt\" > in_1.txt${NC}"
    echo -e "${BRed}echo \"The second line of in_1.txt\" >> in_1.txt${NC}"
    echo -e "${BRed}echo \"The third line of in_1.txt\" >> in_1.txt${NC}"

    colorComment "# Display File"
    echo -e "${BRed}cat in_1.txt${NC}"
    echo -e "The first line of in_1.txt"
    echo -e "The second line of in_1.txt"
    echo -e "The third line of in_1.txt"

    colorComment "Use file as STDIN to the while loop"
    echo -e "${BRed}while read line; do echo \$line | awk '{print \$2}'; done < in_1.txt${NC}"
    echo -e "first\nsecond\nthird"
    echo

    rm *.txt
}

err_demo ()
{
    reset
    colorComment "Make 2 Files"
    echo -e "${BRed}echo \"This line is in a.txt\" > a.txt${NC}"
    echo -e "${BRed}echo \"This line is in b.txt\" > b.txt${NC}"

    colorComment "Show STDERR manipulation"
    echo -e "${BRed}cat a.txt b.txt${NC}"
    echo -e "This line is in a.txt"
    echo -e "This line is in b.txt"
    echo -e "\n${BRed}cat a.txt b.txt c.txt${NC}"
    echo -e "This line is in a.txt"
    echo -e "This line is in b.txt"
    echo -e "cat: c.txt: No such file or directory"
    echo -e "\n${BRed}cat a.txt b.txt c.txt 2>/dev/null${NC}"
    echo -e "This line is in a.txt"
    echo -e "This line is in b.txt"
    echo -e "\n${BRed}cat a.txt b.txt c.txt &> output.txt${NC}"
    echo -e "${BRed}cat output.txt${NC}"
    echo -e "This line is in a.txt"
    echo -e "This line is in b.txt"
    echo -e "cat: c.txt: No such file or directory"

}

redirect ()
{
    reset
    colorComment "Redirect STDIN into the page viewer program \"less\""
    echo -e "${BRed}cat /etc/passwd | less${NC}"

    colorComment "Runs the grep command on the contents(STDOUT of cat) of /etc/passwd, returning the matching line"
    echo -e "${BRed}cat /etc/passwd | grep root${NC}"
    echo -e "root:x:0:0:root:/root:/bin/bash"

    colorComment "Same as above, but passes the line to awk to return only the relevent fields"
    echo -e "${BRed}cat /etc/passwd | grep root | awk -F: '{print \$1\" \"\$3\" \"\$4\" \"\$6}'${NC}"
    echo -e "root 0 0 /root"
}

dev_demo ()
{
    reset
    colorComment "# Shows the association between File Descriptors and STDOUT, STDIN, and STDERR"
    echo -e "${BRed}ls -l /dev/std*${NC}"
    echo -e "lrwxrwxrwx 1 root root 15 Apr 19 16:57 /dev/${BGreen}stderr${NC} -> /proc/self/fd/${BGreen}2${NC}"
    echo -e "lrwxrwxrwx 1 root root 15 Apr 19 16:57 /dev/${BYellow}stdin${NC} -> /proc/self/fd/${BYellow}0${NC}"
    echo -e "lrwxrwxrwx 1 root root 15 Apr 19 16:57 /dev/${BPurple}stdout${NC} -> /proc/self/fd/${BPurple}1${NC}"

}


mainMenu () {

clear

while :
do
    #clear

    echo
    echo "Choose from the following options:"
    echo "----------------------------------"
    echo "  [1] Show /dev/ --> /proc association"
    echo "  [2] Standard Out Demo"
    echo "  [3] Standard In Demo"
    echo "  [4] Standard Err Demo"
    echo "  [5] Pipe Redirection Demo"
    echo
    echo "  [X] to Exit : [C] to Clear the Screen" && echo

    read -p "Choice: " choice
    echo

    case "$choice" in
        1       ) dev_demo
                ;;
        2       ) out_demo
                ;;
        3       ) in_demo
                ;;
        4       ) err_demo
                ;;
        5       ) redirect
                ;;
        [xX]    ) return 0
                ;;
        [cC]    ) clear
                ;;
        *       ) echo "Please enter a valid option"
                ;;
    esac
done
}
# Run the program
mainMenu
